# ppl.pl for Irssi (port of asmodean's /ppl command from skuld3)
# fahren@bochnia.pl /thanks to elluin & lemur/

# Usage: /ppl [-o | -v | -l | -i]
#  To list ops | voices | normal | ircops
# You can mix it with:
# /ppl [-g | -h]
#  To list away / unaway people, or:
# /ppl [-p <n!u@h> | -m <*ircserver*> ]
#  To list people matching n!u@h or using server matching *ircserver*

Irssi::theme_register([
	'ppl_line', '%W$0%n$1$2%B@%n$3$4%C$5%n',
	'ppl_end', '%y>>%n $0 - matched %_$1%_ users (*=%_$2%_ -o=%_$3%_ +v=%_$4%_ +o=%_$5%_)'
]);

use Irssi;
use POSIX qw(strftime);

sub cmd_ppl {

	my ($pars, $server, $chan) = @_;
	my @data = split(/ /, $pars);
	my $window = Irssi::active_win();
	$ppl = "", $ppl{o} = 0, $ppl{v} = 0, $ppl{l} = 0, $ppl{m} = 0, $ppl{i} = 0;
	$ppl{s} = "*", $ppl{h} = "*!*@*";
	$ppl{w} = $window->{width};

	while ($_ = shift(@data)) {
		/^-o$/	and $ppl .= "@", next;
		/^-i$/	and $ppl .= "\\*", next;
		/^-v$/	and $ppl .= "\\+", next;
		/^-l$/	and $ppl .= "X", next;
		/^-g$/	and $ppl = "G" . $ppl, next;
		/^-h$/	and $ppl = "H" . $ppl, next;
		/^-s$/	and $ppl{s} = shift(@data), next;
		/^-p$/	and $ppl{h} = shift(@data), next;
	}
	
	$ppl{c} = $window->get_active_name();
	if ($ppl{c} eq "" || $ppl{c} eq "(status)") {
		Irssi::print("%R>>%n You have to join channel first :\\", MSGLEVEL_CRAP);
		return;
	}
	
	$ppl{s} =~ s/\./\\./g;
	$ppl{s} =~ s/\*/.*/g;

	if (Irssi::settings_get_bool(timestamps)) {
		my $ts_for = Irssi::settings_get_str(timestamp_format);
		$ppl{w} -= (length(strftime($ts_for, localtime)) + 1);
	}
	
	$server->send_raw("WHO :$ppl{c}");
	$server->redirect_event("$ppl{c}", 1,
		"event 315", "redir ppl_end", 1,
		"event 352", "redir ppl_line", 1
	);
}

sub red_ppl_end {
	my ($server, $data) = @_;
	$server->printformat($ppl{c}, MSGLEVEL_CLIENTCRAP, 'ppl_end', $ppl{c}, $ppl{m}, $ppl{i}, $ppl{l}, $ppl{v}, $ppl{o});
}

sub red_ppl_line {
	my ($s1, $data, $s2) = @_;
	my ($c1, $c2, $user, $host, $server, $nick, $mode, $hops, $c3) = split(/ /, $data, 10);
	
	if ($ppl =~ /X/) {
		if ($mode =~ /@/) {return}
	} elsif (!($mode =~ /$ppl/)) {return}

	return if !($s1->mask_match($ppl{h}, $nick, $user, $host));
	return if !($server =~ /$ppl{s}/);			

	$ppl{m}++;
	$mode =~ /\*/ && $ppl{i}++;
	if ($mode =~ /@/) {$ppl{o}++
	} elsif ($mode =~ /\+/) {$ppl{v}++
	} else {$ppl{l}++}

	$mode = sprintf("%-2.2s", $mode);
	$nick = sprintf("%.9s", $nick);
	$nick .= "." x (11 - length($nick));
	$user = sprintf("%.11s", $user);
	$user = "." x (11 - length($user)) . $user;
	$host = sprintf("%.28s", $host);
	$server = "." x ($ppl{w} - 27 - length($host) - length($server)) . $server;
	$hops =~ s/^://;
	
	$s1->printformat($ppl{c}, MSGLEVEL_CLIENTCRAP, 'ppl_line', "$mode", "$nick", "$user", "$host", "$server", "$hops");
}

Irssi::command_bind("ppl", "cmd_ppl");
Irssi::signal_add("redir ppl_line", "red_ppl_line");
Irssi::signal_add("redir ppl_end", "red_ppl_end");
Irssi::print("Loaded ppl.pl");
