# $Id: stat.pl 1.1 2002/03/23 01:55:50 morrow Exp $
# This script is for parsing the kline, iline, stats L and user trace output.
# With a given pattern it shows only matching lines.
#
# Remember: You can only use "/stat -k" if you are not an IRC operator.
# All other options require you to be /oper-ed.
#
# Examples:
# /stat -u 1005         shows you all lines that contain the string '1005'
#                       useful for showing all clients in the given class
# /stat -u unixshells   the same for hostname matching. Shows all
#                       clients that come from *unixshells*
# /stat -l 195.180.4    shows all users that come from an IP address
#                       that is matching *195.180.4*
# /stat -k bofh.com     shows all klines that contain 'bofh.com'
#
# With -k and -i you can add a server name to get the K-lines and
# I-lines from a remote server:
#
# /stat -k bofh.com uni-erlangen.de     Shows all klines on server
#                       uni-erlangen.de, that contain 'bofh.com'
#
#
# This script is the irssi version of frank altpeters stat.irc for epic.
# Just the idea was taken from frank, so no need to keep the GPL. :o) 
#

use Irssi;
use Irssi::Irc;
use strict;
use vars qw($VERSION %IRSSI);

$VERSION = "0.05";
%IRSSI = (
    authors     => 'Adrian Woizik',
    contact     => 'morrow@unfug.org',
    name        => 'stat.pl',
    description => 'stats.parser',
    license     => 'BSD'
);

my ($type,$typenr, $count, $hits, $search, $window);
sub stat {
	my ($argv, $server) = @_;


       	$count = 0;
       	$hits = 0;
	my $rserver;
	$window = Irssi::active_win();	
	
	if 	($argv =~ /^-i\s(.*)$/) {
		$search = $1;
		if ($search =~ /(.*)\s(\S*)$/) {
			$search = $1;
			$rserver = $2;
		}
        	$type = "Ilines";
		$typenr = '215';

		$window->print("Searching $type for $search $rserver ");

		Irssi::signal_add('event 215', 'event_stats');
		Irssi::signal_add('event 219', 'event_stats_end');
		$server->send_raw("stats i $rserver");
		return;
	} elsif ($argv =~ /^-u\s(.*)$/) {
                $search = $1;

                $type = "users";
                $typenr = '';

		unless($server->{server_operator}) {
			$window->print("permission denied - not an IRC Operator");
			return;
		}
                $window->print("Searching $type for $search");

                Irssi::signal_add('event 203', 'event_stats');
                Irssi::signal_add('event 204', 'event_stats');
                Irssi::signal_add('event 205', 'event_stats');
                Irssi::signal_add('event 206', 'signal_stop');
                Irssi::signal_add('event 207', 'signal_stop');
                Irssi::signal_add('event 208', 'signal_stop');
                Irssi::signal_add('event 209', 'signal_stop');
                Irssi::signal_add('event 262', 'event_trace_end');
                $server->send_raw("trace");
		return;
	} elsif ($argv =~ /^-l\s(.*)$/) {
                $search = $1;
                $type = "Llines";
		$typenr = '211';

		unless($server->{server_operator}) {
			$window->print("permission denied - not an IRC Operator");
			return;
		}
                $window->print("Searching $type for $search");

                Irssi::signal_add('event 211', 'event_stats');
                Irssi::signal_add('event 219', 'event_stats_end');
                $server->send_raw("stats L");
		return;
	} elsif ($argv =~ /^-k\s(.*)$/) {
	        $search = $1;
                if ($search =~ /(.*)\s(\S*)$/) {
                        $search = $1;
                        $rserver = $2;
                }

                $type = "K-lines";
		$typenr = '216';

                $window->print("Searching $type for $search");

                Irssi::signal_add('event 216', 'event_stats');
                Irssi::signal_add('event 219', 'event_stats_end');
                $server->send_raw("stats K");
		return;
	} 
	$window->print("USAGE: /stat \[-i|-k|-u|-l\] <regexp> [server]");
}
sub event_stats {
        my ($server, $data, $nick, $address) = @_;
        my ($target, $text) = $data =~ /^(\S*)\s+(.*)$/;
       
        #$window->print("server: $server data: $data : \@: @_");
        if ($text =~ /$search/) {
                $text =~ s/ /:/g;
                $text =~ s/<NULL>//g;
                $window->print("$text");
                $hits++;
        }
        $count++;
        Irssi::signal_stop();
}

sub signal_stop {
	my ($server, $data, $nick, $address) = @_;
	Irssi::signal_stop();
}

sub event_stats_end {
	my ($server, $data, $nick, $address) = @_;

	$window->print("found $search $hits times in $count $type.");
	Irssi::signal_remove("event $typenr", 'event_stats');
	Irssi::signal_remove('event 219', 'event_stats_end');
}
sub event_trace_end {
        my ($server, $data, $nick, $address) = @_;

        $window->print("found $search $hits times in $count $type.");
 	Irssi::signal_remove('event 203', 'event_stats');
 	Irssi::signal_remove('event 204', 'event_stats');
 	Irssi::signal_remove('event 205', 'event_stats');
        Irssi::signal_add('event 206', 'Irssi::signal_stop()');
        Irssi::signal_add('event 207', 'Irssi::signal_stop()');
        Irssi::signal_add('event 208', 'Irssi::signal_stop()');
        Irssi::signal_add('event 209', 'Irssi::signal_stop()');
        Irssi::signal_remove('event 262', 'event_trace_end');
}

Irssi::command_bind("stat", 'stat');
